# jspsych-iat plugin

A list of images or HTML-formatted content is given as the stimulus. There are two or more labels associated with each stimulus and two different key presses associated with the labels. For example, you would press 'f' for the right category lable and 'j' for the left category lable. Throughout the experiment, data will be collected such as the number of correct key presses, the time it takes to get a response, etc. Here is a link if you want a better understanding of what the IAT plugin is:  https://implicit.harvard.edu/implicit/iatdetails.html

Because this plugin can display any HTML content, it is quite versatile. It can be used for any situation in which the response generated by the subject is a single keystroke.

## Parameters

This table lists the parameters associated with this plugin. Parameters with a default value of *undefined* must be specified. Other parameters can be left unspecified if the default value is acceptable.

Parameter | Type | Default Value | Description
----------|------|---------------|------------
stimulus | string | *undefined* | The stimulus to display. Either HTML-formatted, or the path to an image.
is_html | boolean | false | If `stimulus` is an HTML-formatted string, this parameter needs to be set to `true`.
prompt | string | "" | This string can contain HTML markup. Any content here will be displayed below the stimulus. The intention is that it can be used to provide a reminder about the action the subject is supposed to take (e.g. which key to press).
image_when_wrong | string | *undefined* | Displays image feedback when user presses wrong key. Only needed when display_feedback == true. 
wrong_image_name | string | 'wrong image' | The name associated with image_when_wrong. 
display_feedback | boolean | false | If true, then image_when_wrong and wrong_image_name is required. If false, timing_response is needed and trial will continue automatically. 
left_category_key | string | 'E' | Key press that is associated with the left_category_label. 
right_category_key | string | 'I' | Key press that is associated with the right_category_label. 
left_category_label | string | ['left'] | An array that contains the words/labels associated with a certain stimulus. The labels are aligned to the left side of the page.
right_category_label | string | ['right'] | An array that contains the words/labels associated with a certain stimulus. The labels are aligned to the right side of the page. 
stim_key_association | string | 'undefined' | Inputs are either 'left' or 'right'. It will associate the stimulus with the key presses on the left or right side of the page(left_category_key or right_category_key). 
key_to_move_forward | array of characters | [jsPsych.ALL_KEYS] | This array contains the characters the subject is allowed to press to move on to the next trial if their key press was incorrect and feedback was displayed. Can also have 'other key' as an option which will only allow the user to select the right key to move forward. 
timing_response | numeric | -1 | How long to wait for the subject to make a response before ending the trial in milliseconds. If the subject fails to make a response before this timer is reached, the the subject's response will be recorded as -1 for the trial and the trial will end. If the value of this parameter is -1, then the trial will wait for a response indefinitely.
response_ends_trial | boolean | true | If true, then the trial will end whenever the subject makes a response (assuming they make their response before the cutoff specified by the `timing_response` parameter). If false, then the trial will continue until the value for `timing_response` is reached. You can use this parameter to force the subject to view a stimulus for a fixed amount of time, even if they respond before the time is complete.

## Data Generated

In addition to the [default data collected by all plugins](overview#datacollectedbyplugins), this plugin collects the following data for each trial.

Name | Type | Value
-----|------|------
stimulus | string | Either the path to the image file or the string containing the HTML formatted content that the subject saw on this trial.
key_press | numeric | Indicates which key the subject pressed. The value is the [numeric key code](http://www.cambiaresearch.com/articles/15/javascript-char-codes-key-codes) corresponding to the subject's response.
rt | numeric | The response time in milliseconds for the subject to make a response. The time is measured from when the stimulus first appears on the screen until the subject's response.
correct | boolean | Boolean of whether the user's key press was correct for the given image or incorrect. 

## Examples

```javascript 
var trial_block = {
type: 'iat',
stimulus: 'img/blue.png',
stim_key_association: 'left', 
is_html: false,
image_when_wrong: 'img/redX.png',
wrong_image_name: 'red X',
display_feedback: true, 
timing_response: 3000, //Only if display_feedback is false 
left_category_key: 'E',
right_category_key: 'I',
left_category_label: ['OLD'],
right_category_label: ['YOUNG'],
key_to_move_forward: [jsPsych.ALL_KEYS],
response_ends_trial: true
}

var timeline = [];
timeline.push(trial_block);

jsPsych.init({
timeline: timeline,
on_finish: function() {
jsPsych.data.displayData();
}
});
